<?php

namespace apexl\Io\modules\component\Collections;

use apexl\Io\modules\component\Component\Component;
use apexl\Io\modules\component\Interfaces\ComponentInterface;
use Arrayy\Arrayy;
use Arrayy\Collection\AbstractCollection;

/**
 * @extends Arrayy<array-key, Component>
 * @method ComponentCollection add(Component $value, $key = null)
 */
final class ComponentCollection extends AbstractCollection
{
    public function getType(): string
    {
        return ComponentInterface::class;
    }
}
