<?php

namespace apexl\Io\modules\component\Collections;

use apexl\Io\modules\component\Component\ComponentProperty;
use Arrayy\Collection\AbstractCollection;

final class ComponentPropertyCollection extends AbstractCollection
{
    public function getType(): string
    {
        return ComponentProperty::class;
    }
}
